'FFSubmenu=Tools
'FFName=ImageBackup

' ********************************************************************
' ImageBackup (Skript fr FixFoto)
' Funktion:	Erstellung und Aktualisierung von Backups eines Ordners mit Unterordner
'
' Copyright Harald Maeckler, Dsseldorf 2014 - 2016
' ********************************************************************
'
' Version 1.03 vom 03.05.2016	- Bug bei Option "mit Struktur" beseitigt
' Version 1.02 vom 15.08.2014	- beim Backupverzeichnis sind jetzt auch Unterordner zugelassen
'								- Skript-URL gem Forenberarbeitung gendert
' Version 1.01 vom 13.05.2014	- Option: automatisch aktuelles Verzeichnis
'								- nach Backup muss das Skript manuell beendet werden
' Version 1.00 vom 01.05.2014	- Betaversion
' ********************************************************************
'
'** Start Kurzbeschreibung *******************************************
' ImageBackup / ab Version 1.03
' ===================
' 
' Das Skript ist nur in der Umgebung der Software FixFoto lauffhig.
' Es kann nach dem Import in der Kategorie "Tools" aufgerufen werden.
'
' Funktion
' =====
' Das Skript dient der Erstellung und Aktualisierung von Backups eines Ordners mit
' Unterordner inklusive ihrer Dateien. Beim ersten Backup werden alle Dateien der
' gewhlten Quelle zum gewhlten Ziel kopiert und dabei das Archivbit jeder Datei
' zurck gesetzt. Werden vorhandene Dateien bearbeitet, setzt das WINDOWS-
' Betriebssystem das Archivbit neu. Beim nchsten Backup bei gleicher Quelle
' und demselben Ziel werden nur noch alle neuen Dateien und die bearbeiteten
' Dateien mit gesetztem Archivbit kopiert.
' 
' Quellordner
' =======
' Man kann wahlweise mit Anklicken 'Aktuelles_Verzeichnis' den gerade im Computer-
' verzeichnis aktiven Ordner auswhlen oder mit Mausklick auf 'Beliebiges_Ver-
' zeichnis' einen Dateiexplorer ffnen und dort ein beliebigen Ordner zum Quell-
' ordner bestimmen.
'
' Bei aktivierter Option 'automatisch aktuelles Verzeichnis verwenden' wird mit
' 'Start' immer das aktuelle Computerverzeichnis  als Quelle verwendet.
'
' Standardmig wird das Backup mit dem Namen des Ordners als Unterordner des
' Backup-Ziels erstellt. Bei Aktivierung der Option 'Backup mit Struktur der
' bergeordneten Ordner' werden im Backup alle bergeordneten Ordner strukturell
' dargestellt. Nheres dazu wird im Kapitel 'Backup-Ziel' erlutert.
'
' Backup-Ziel
' =======
' Es werden alle dem System zur Verfgung stehenden externen und internen Lauf-
' werke zur Auswahl angeboten mit Ausnahme des CD/DVD-Laufwerks. Das ausgewhlte
' Laufwerk und das vorgebbare Verzeichnis zusammen ergeben das Backup-Ziel.
'
' Das Backup-Ziel wird bei deaktivierter Option 'Backup mit Struktur der berge-
' ordneten Ordner' ergnzt um den Namen des Quellordners. Daraus ergibt sich bei-
' spielsweise
' 		C:\Backup_Image\2014_03 Lantzscher Park
'
' Vorteil:
' --------
' Auch wenn der Quellordner verschoben wird, ist eine Aktualisierung des Backups
' mglich.
'
' Nachteil:
' ----------
' Wird spter einer der Unterordner des Quellordners als neue Quelle fr ein
' Backup ausgewhlt, wird keine Aktualisierung eines vorhandenen Backups vorge-
' nommen, sondern ein neues Backup angelegt. Dabei besteht die Gefahr, dass bei
' gleichem Namen Backups berschrieben werden. Daher sollte in jedem Fall das
' Zielverzeichnis gendert werden.
'
' Bei aktivierter Option "Backup mit Struktur der bergeordneten Ordner" wird
' das Backup-Ziel ergnzt um den Namen des Quellordners mit allen bergeordneten
' Ordnernamen. Daraus ergibt sich beispielsweise
' 		C:\Backup_Image\Eigene Bilder\2014_03 Lantzscher Park
'
' Vorteil:
' --------
' Wird spter einer der Unterordner des Quellordners als neue Quelle fr ein
' Backup ausgewhlt, wird dieser Ordner im vorhandenen Backup aktualisiert.
'
' Nachteil:
' ----------
' Das Backup wird geringfgig umfangreicher und man muss beim Durchforsten eines
' Backups zum Auffinden eines bestimmten Ordners die gesamte Struktur durchsuchen.
' Wird der Quellordner verschoben, ist eine Aktualisierung des Backups nicht mehr
' mglich. Es muss ein neues Backup angelegt werden.
'
' Empfehlung:
' ---------------
' Hat man eine klare Ordnerstruktur, die in der Regel nur ergnzt aber nicht ge-
' ndert wird, ist unbedingt die Aktivierung der Option 'Backup mit Struktur der
' bergeordneten Ordner' zu empfehlen.
'
' Weitere Befehle
' ==========
'
' Fragezeichen
' ----------------
' Mit dem '?' wird die vorliegende Kurzbeschreibung aufgerufen.
'
' Update
' ---------
' Unten links wird neben dem '?', das dem Aufruf der vorliegenden Kurzbeschrei-
' bung dient, die Versionsnummer des installierten Skripts angezeigt, z.B.
' 'V 1.00'. Wird der daneben liegende Button 'Update' angeklickt, wird per Inter-
' netverbindung versucht auf die Downloadseite des Skripts zuzugreifen und die
' dort aktuell angebotene Versionsnummer auszulesen. Ist diese aktueller, wird
' angeboten die Internetseite aufzurufen. Bei Zustimmung wird das Skript beendet
' und die Downloadseite im Standardbrowser angezeigt. Jetzt kann manuell die
' neuere Version herunter geladen und installiert werden.
'
' Beenden
' -----------
' Mit 'Beenden' und dem Kreuz oben links im roten Feld wird das Skript beendet.
'
' Start
' ------
' Bei Bettigung von 'Start' werden die Einstellungen berprft und ggf. auf
' mangelhaften Speicherplatz und fehlerhafte Angaben hingewiesen. Dann wird mit
' Anzeigen von Quelle und Ziel gemeldet, ob ein Backup oder die Aktualisierung
' eines vorhandenen Backups durchgefhrt werden soll. Nach Besttigung mit 'OK'
' wird der Vorgang gestartet.
' Der Fortschritt wird mit einem grnen Balken angezeigt. Eine Unterbrechung ist
' durch Bettigung von 'Backspace' auf der Tastatur jederzeit mglich. Dabei muss
' die Taste festgehalten werden bis angezeigt wird, wie viel Dateien bereits be-
' arbeitet wurden. Man kann dann entscheiden, ob Vorgang wirklich abgebrochen
' oder fortgesetzt werden soll.
'
' Skript lschen
' -----------------
' Wird die Entfernung des Skripts gewnscht, kann mit der Tastenkombination Alt + D
' eine Lschroutine aufgerufen werden. Es wird geprft, ob ein dazu gehrendes zip-
' Archiv im Skriptordner abgelegt ist. Wenn ja, wird vor dem Lschen angeboten, davon
' eine Sicherheitskopie an einem beliebigen Speicherort abzulegen. Dann werden nach-
' einander die Skriptdatei, die in der Registry dazugehrenden Daten und eventuelle
' Zusatzdateien gelscht. Abschlieend wird die FixFotoScript.ini bereinigt.
' 
' Nach manueller Durchfhrung einer Automatischen Skriptkonfiguration ist das Skript
' mit allen Komponenten aus der FixFoto-Umgebung entfernt.
' 
' 
' Benutzung des Skripts auf eigene Gefahr!
' 
' Copyright Harald Maeckler, Dsseldorf
' 
'** Ende Kurzbeschreibung ***********************************************

'**Start Encode**
Option Explicit

const SkriptVersion = "1.03"								'Updatekonstanten und Variable
const SkriptURL = "http://www.ffsf.de/resources/imagebackup.251/"
Dim UrlVersion,UrlV_Zahl,SkriptV_Zahl,SkriptEnde

Dim SkriptName,FF_Skriptpfad
const Skript = "ImageBackup"								'Skriptname
const RegSchluessel = "FF_ImageBackup"						'Registryschlssel

Dim Kurzbeschreibung, KBvorhanden, LaengeMax, Abbruch		'Kurzbeschreibung
const ForReading = 1, ForWriting = 2
Abbruch = false

Dim objFS,WshShell											'Objekte
Set objFS = CreateObject("Scripting.FileSystemObject")
Set WshShell = CreateObject("WScript.Shell")

Dim SkriptKill,Antwort,TempFile	 : SkriptKill = false		'Lschroutine

'Variable fr vorliegendes Skript
Dim strPathOriginal,Quelle,strDrive,strPathKopie,flag_Auto,flag_Struktur
Dim element,LWarray,AnzGes
Dim intKopiert,intAktuell,t1,t2

'------------------------------------------------------------------------
Set objFS = CreateObject("Scripting.FileSystemObject")
Set WshShell = CreateObject("WScript.Shell")

call Main

Set objFS = Nothing
Set WshShell = Nothing
'------------------------------------------------------------------------
Sub Main
Dim Anzeige

on error resume next							'Kontrolle ob Skript in FF-Umgebung gestartet wurde
	FF_Skriptpfad = FF_GetScriptPath
 	if err > 0 then
		msgbox "Das Skript funktioniert nur in der Programmumgebung von FixFoto!",vbCritical,"Abbruch"
		exit sub
	end if
on error goto 0

'Originalskript
if objFS.FileExists(FF_GetScriptPath & "\" & Skript & ".vbs") then
	SkriptName = Skript & ".vbs"
elseif objFS.FileExists(FF_GetScriptPath & "\" & Skript & ".vbe") then
	SkriptName = Skript & ".vbe"
else
	msgbox "Das Skript wurde umbenannt!",vbCritical,"Abbruch"
	exit sub
end if

'** Start Dialog ImageBackup **
FF_AddDialog "ImageBackup",144,170
FF_AddControl "ImageBackup","Quellordner","GROUP",6,6,132,64
FF_AddControl "ImageBackup","Backup-Ziel","GROUP",6,74,132,32
FF_AddControl "ImageBackup","strPathOriginal","SLEDIT",11,16,122,10
FF_SetControlStyle "ImageBackup","strPathOriginal",2048
FF_AddControl "ImageBackup","flag_Auto","BOOL",11,46,8,8
FF_SetControlStyle "ImageBackup","flag_Auto",16384
FF_AddControl "ImageBackup","automatisch aktuelles Verzeichnis verwenden","STATIC",21,45,116,10
FF_AddControl "ImageBackup","flag_Struktur","BOOL",11,58,8,8
FF_AddControl "ImageBackup","Backup mit Struktur der bergeordneten Ordner","STATIC",21,57,116,10
FF_AddControl "ImageBackup","Aktuelles Verzeichnis","BUTTON",11,30,60,11
FF_AddControl "ImageBackup","Beliebiges Verzeichnis","BUTTON",73,30,60,11
FF_AddControl "ImageBackup","Laufwerk","STATIC",11,80,25,10
FF_AddControl "ImageBackup","strDrive","COMBO",11,90,60,10
FF_AddControl "ImageBackup","Verzeichnis","STATIC",73,80,30,10
FF_AddControl "ImageBackup","strPathKopie","SLEDIT",73,90,60,10
FF_AddControl "ImageBackup","Fortschritt","PROGRESS",6,112,132,4
FF_AddControl "ImageBackup","?","BUTTON",7,128,11,11
FF_AddControl "ImageBackup","|","STATIC",22,128,20,11
FF_SetControlStyle "ImageBackup","|",1
FF_AddControl "ImageBackup","Update","BUTTON",45,128,28,11
FF_AddControl "ImageBackup","Vorgang abrechen mit","STATIC",81,122,54,11
FF_AddControl "ImageBackup","Backspace","BUTTON",82,132,30,9
FF_AddControl "ImageBackup","- Taste","STATIC",113,131,20,11
FF_AddControl "ImageBackup","Beenden","BUTTON",26,152,33,11
FF_AddControl "ImageBackup","Start","BUTTON",81,152,33,11
FF_AddControl "ImageBackup","&d","BUTTON",1,1,1,1
'** End Dialog ImageBackup **

'Laufwerke eintragen
LWarray = split(Laufwerke,",")
for each element in LWarray
	FF_SetControl "ImageBackup","strDrive",element
next

'Registry
call RegistryLesen

if flag_Auto = 1 then
	FF_SetControl "ImageBackup","strPathOriginal",FF_GetImagePath
	FF_EnableControl "ImageBackup","Aktuelles Verzeichnis", false
	FF_EnableControl "ImageBackup","Beliebiges Verzeichnis", false
end if

'Tooltips
if Version_OK("3.30.06") then
	FF_SetControlTooltip "ImageBackup","Aktuelles Verzeichnis", "" &_
						 "bernahme des aktuellen Computerverzeichnis"
	FF_SetControlTooltip "ImageBackup","Beliebiges Verzeichnis", "" &_
						 "ffnen des Explorers zur" & vbNewLine &_
						 "Auswahl des Quellordners"
	FF_SetControlTooltip "ImageBackup","?", "" &_
						 "Anzeige der Kurzbeschreibung als PDF-Datei," & vbNewLine & _
						 "vorausgesetzt es ist ein PDF-Reader installiert." & vbNewLine & _
						 "Anderenfalls wird eine gekrzte Textversion angezeigt."
	FF_SetControlTooltip "ImageBackup","Update", "" &_
						 "Prfung ob eine neue Version vorliegt."
	FF_SetControlTooltip "ImageBackup","Backspace", "" &_
						 "Vorzeitiger Abbruch mglich durch" & vbNewLine &_
						 "Bettigung der 'Backspace'-Taste."
	FF_SetControlTooltip "ImageBackup","Beenden", "" &_
						 "Skript beenden"
	FF_SetControlTooltip "ImageBackup","Start", "" &_
						 "Start des Backups"
end if

if Version_OK("3.40.52") then FF_SetTooltipDelay 2500
FF_ActivateControl "ImageBackup","&d"
FF_SetDialogTimer "ImageBackup", 100

do

'Tooltips
	if Version_OK("3.30.06") then
		FF_SetControlTooltip "ImageBackup","strPathOriginal", "" &_
						 	FF_GetControl("ImageBackup","strPathOriginal")
	end if

	Select Case FF_ShowDialog ("ImageBackup")
	Case "&d"
		if len(Skriptname) = 0 then
			msgbox "Das Skript '" & OriginalSkript & "'konnte nicht gefunden werden!" & vbNewLine & _
				   "Mglicherweise wurde der Name gendert.",vbExclamation,"Keine Lschung mglich"
		else
			Antwort = msgbox("Soll das Skript '" & Skriptname & "'" & vbNewline & _
							 "wirklich gelscht werden?", _
							 vbYesNo + vbQuestion,"Sicherheitsabfrage")
			if Antwort = vbYes then
				SkriptKill = true
				exit do
			end if
		end if
	Case "TIMER"
		FF_SetControl "ImageBackup" ,"|", "V " & Skriptversion
		FF_SetDialogTimer "ImageBackup", 0
	Case "Aktuelles Verzeichnis"
		FF_SetControl "ImageBackup","strPathOriginal",FF_GetImagePath
	Case "Beliebiges Verzeichnis"
		FF_SetCurrentPath FF_GetControl("ImageBackup","strPathOriginal")
		Quelle = FF_EnterPath(false,"Wahl des Quellordners")
		if len(Quelle) > 2 then
			FF_SetControl "ImageBackup","strPathOriginal",Quelle
		end if
	Case "flag_Auto"
		flag_Auto = FF_GetControl("ImageBackup","flag_Auto")
		if flag_Auto = 1 then
			FF_SetControl "ImageBackup","strPathOriginal",FF_GetImagePath
			FF_EnableControl "ImageBackup","Aktuelles Verzeichnis", false
			FF_EnableControl "ImageBackup","Beliebiges Verzeichnis", false
		else
			FF_EnableControl "ImageBackup","Aktuelles Verzeichnis", true
			FF_EnableControl "ImageBackup","Beliebiges Verzeichnis", true
		end if
	Case "Update"
		SkriptEnde = false
		call Update
		if SkriptEnde then exit do
	Case "?"
		'FF_SetControl "ImageBackup","Fortschritt",65	'############## nur fr Screenshot
		Anzeige = chr(34) & FF_GetScriptPath & "\MeckiScript\ImageBackup\ImageBackup.pdf" & chr(34)
		on error resume next
		WshShell.Run Anzeige,1,false
		if err <> 0 then
			msgbox "Die Kurzbeschreibung konnte nicht als PDF-Datei geffnet werden." & vbNewLine & _
				   "Die Datei ist gelscht oder es ist kein PDF-Reader installiert (?)." & vbNewLine & vbNewLine &_
				   "Ersatzweise erfolgt eine Textanzeige!",vbExclamation,"Hinweis"
			err.clear
			
			if not KBvorhanden then	call KurzbeschreibungLesen
			if KBvorhanden then
				call KurzbeschreibungAnzeigen
			else
				msgbox "Keine Kurzbeschreibung gefunden!",vbExclamation,"Meldung"
			end if
			
		end if
		on error GoTo 0
	Case "Start"
		if flag_Auto = 1 then FF_SetControl "ImageBackup","strPathOriginal",FF_GetImagePath
		call WerteAuslesen
		call Kontrolle
		if not Abbruch then
			call Vorbereitung
'			exit do				'testweise auskommentiert
		end if
	Case "CANCEL","Beenden"
		exit do
	Case Else
	End Select
loop

call RegistrySchreiben
FF_CloseDialog ("ImageBackup")

if SkriptKill then	'ggf. Skript und Zubehr komplett entfernen
	call KillSkript
end if

End Sub
'------------------------------------------------------------------------
Sub Vorbereitung
Dim objStart,f,Qgroesse,ZLWgroesse,Zgroesse,Restgroesse,Dauer,msg,strPath,pos
'Msgbox "Orig. = " & strPathOriginal & " || Kopie = " & strPathKopie
if flag_Struktur = 0 then
	'# Quellordner in Zielordner einbinden
	Set f = objfs.GetFolder(strPathOriginal)
	strPathKopie = left(strdrive,2) & "\" & strPathKopie & "\" & f.Name
'Msgbox "ohne Str. = " & strPathKopie & " || Name = " & f.Name
else
	'# Struktur des Quellordners in Zielordner einbinden
	pos = instr(strPathOriginal,"\")	'### bisher InStrRev
	strPath = mid(strPathOriginal,pos)
	strPathKopie = left(strdrive,2) & "\" & strPathKopie & strPath
'Msgbox "mit Str. = " & strPathKopie & " || Pfad = " & strPath
end if

'### Speichergre berprfen

'zur Verfgung stehende Gre des Ziellaufwerks
Set f = objfs.GetDrive(objfs.GetDriveName(objfs.GetAbsolutepathname(strPathKopie)))
ZLWgroesse = f.AvailableSpace
'MsgBox SpeicherGroesse(ZLWgroesse),,"ZLWgroesse"

'Quellgre und Anzahl Dataien
Set f = objfs.GetFolder(strPathOriginal)
Call ShowSubfolderCount(f)
'MsgBox AnzGes & " Dateien",vbInformation,"Info"

Qgroesse = f.size
'MsgBox SpeicherGroesse(Qgroesse),,"Qgroesse"

'Ziel- und Restgroesse
If objfs.FolderExists(strPathKopie) Then
	Set f = objfs.GetFolder(strPathKopie)
	Zgroesse = f.size
	msg = "Die Aktualisierung "
Else
	Zgroesse = 0
	msg = "Die Erzeugung "
End If
'MsgBox SpeicherGroesse(Zgroesse),,"Zgroesse"
Restgroesse = ZLWgroesse - (Qgroesse - Zgroesse)
'MsgBox SpeicherGroesse(Restgroesse),,"Restgroesse"

If Restgroesse > 1000 Then
	Antwort = MsgBox(msg & "des BackUps von" & vbNewLine & "  " &  strPathOriginal & vbNewLine & _
		"im Zielordner" & vbNewLine & "  " & strPathKopie & vbNewLine & "kann gestartet werden." & vbNewLine & vbNewLine & _
		"Der Vorgang kann eine Weile dauern.", vbOKCancel, "Backup")
	if Antwort = vbCancel then Exit Sub
Else
	MsgBox "Der zur Verfgung stehende Speicher von " & SpeicherGroesse(ZLWgroesse) & vbNewLine & _
		"reicht fr das Backup nicht aus!",vbExclamation,"Abbruch"
End If


'### BackUp-Start
Call CreateFolder(strPathKopie)
intKopiert = 0
intAktuell = 0

Set objStart = objfs.GetFolder(strPathOriginal)
t1 = Timer
call BackupFiles(objStart)	'BackUp anlegen
if Abbruch then exit sub
t2 = Timer

msg = intKopiert & " Dateien neu archiviert, " & intAktuell & " waren unverndert." & vbNewLine & vbNewLine
	Dauer = t2 - t1
	If Dauer > 600 Then
		Dauer = FormatNumber(Dauer / 60, 3) & " Minuten"
	Else
		Dauer = FormatNumber(Dauer, 2) & " Sekunden"
	End If
msg = msg & "BackUp-Dauer: " & Dauer & vbNewLine
msg = msg & "Datenvolumen: " & FormatNumber(objfs.GetFolder(strPathKopie).Size / 1024 ^ 2,1) & " MB"

msgBox msg,vbInformation,"BackUp beendet"
FF_SetControl "ImageBackup","Fortschritt",0

End Sub
'------------------------------------------------------------------------
Sub BackupFiles(ByVal objOrdner)
Dim strZielOrdner,objDatei,strZiel,doCopy,objUnterordner
Abbruch = false

strZielOrdner = strPathKopie & Mid(objOrdner.PATH, Len(strPathOriginal) + 1)
'Msgbox "strPathKopie = " & strPathKopie & vbNewLine & "objOrdner.PATH = " & objOrdner.PATH & vbNewLine & "Len(strPathOriginal) + 1 = " & Len(strPathOriginal) + 1 & vbNewLine & "strZielOrdner = " & strZielOrdner
On Error Resume Next
If Not objfs.FolderExists(strZielOrdner) Then
  objfs.CreateFolder strZielOrdner
End If
If Err.Number > 0 Then
	msgBox "Der Zielordner ist nicht generierbar." & vbNewLine & _
			"berprfen, ob das BackUp-Laufwerk existiert!", vbExclamation, "Abbruch"
	Abbruch = true
	Exit Sub
End If
On Error Goto 0

For Each objDatei In objOrdner.files
	strZiel = objfs.BuildPath(strZielOrdner, objDatei.name)

	doCopy = False

	If ((objDatei.Attributes And 32) = 32) Then
		doCopy = True
	ElseIf Not objfs.FileExists(strZiel) Then
    	doCopy = True
	End If

	If doCopy Then
    	objDatei.Copy strZiel

		objDatei.Attributes = objDatei.Attributes And Not 32
		intKopiert = intKopiert + 1
	Else
    	intAktuell = intAktuell + 1
	End If

	'### Fortschritt
	FF_SetControl "ImageBackup","Fortschritt",(intKopiert+intAktuell)/AnzGes*100

	'### Abbruch?
	if FF_CheckKey() = 8 then 	'8 = Backspace
		Antwort = msgbox("Von " & AnzGes & " Dateien wurden bisher " & (intKopiert+intAktuell) & _
			" Dateien berprft." & vbNewLine & vbNewLine & _
			"Soll das Backup abgebrochen werden?",vbYesNo or vbQuestion,"Abfrage")
		if Antwort = vbYes then exit sub
	end if

Next

For Each objUnterordner In objOrdner.subfolders
	BackupFiles objUnterordner	'Rekursion
Next
   
End Sub
'------------------------------------------------------------------------
Sub WerteAuslesen

strPathOriginal = FF_GetControl("ImageBackup","strPathOriginal")
flag_Auto = FF_GetControl("ImageBackup","flag_Auto")
flag_Struktur = FF_GetControl("ImageBackup","flag_Struktur")
strDrive = FF_GetControl("ImageBackup","strDrive")
strPathKopie = FF_GetControl("ImageBackup","strPathKopie")
if left(strPathKopie,1) = "\" then
	strPathKopie = mid(strPathKopie,2)
	FF_SetControl "ImageBackup","strPathKopie",strPathKopie
end if

End Sub
'------------------------------------------------------------------------
Sub Kontrolle
Abbruch = false

if not objFS.FolderExists(strPathOriginal) then
	msgbox "Der Quellordner " & vbNewLine & "  " & strPathOriginal & vbNewLine &_
		"existiert nicht!",vbExclamation,"Hinweis"
	Abbruch = true : exit sub
end if

if instr(Laufwerke,strDrive) = 0 then
	msgbox "Das Laufwerk " & strDrive & vbNewLine &_
		"wurde nicht gefunden!",vbExclamation,"Hinweis"
	Abbruch = true : exit sub
end if

if len(trim(strPathKopie)) = 0 then
	msgbox "Es muss ein Backup-Verzeichnis"& vbNewLine &_
		"eingegeben werden!",vbExclamation,"Hinweis"
	Abbruch = true : exit sub
end if

if not Zeichentest_OK(strPathKopie)then
	MsgBox "Das angegebene Backup-Verzeichnis ist ungltig." & vbNewLine &_
		"Die Zeichen /, :, *, ?, " & Chr(34) & ", <, >, |" & vbNewLine & _
		"drfen nicht verwendet werden!",vbExclamation,"Hinweis"
	Abbruch = true : exit sub
end if

End Sub
'------------------------------------------------------------------------
Sub RegistryLesen
'Schlsselname = HKEY_CURRENT_USER\Software\Joachim Koopmann Software\FixFoto\Script\ & RegSchluessel

'### Werte auslesen

strPathOriginal = FF_GetProfile (RegSchluessel,"strPathOriginal")
if len(strPathOriginal) = 0 then strPathOriginal = FF_GetImagePath

flag_Auto = FF_GetProfile (RegSchluessel,"flag_Auto")
if len(flag_Auto) = 0 then flag_Auto = 0

flag_Struktur = FF_GetProfile (RegSchluessel,"flag_Struktur")
if len(flag_Struktur) = 0 then flag_Struktur = 1

strDrive = FF_GetProfile (RegSchluessel,"strDrive")

strPathKopie = FF_GetProfile (RegSchluessel,"strPathKopie")
if len(strPathKopie) = 0 then strPathKopie = "Backup_Image"

'### Werte eintragen

if objFS.FolderExists(strPathOriginal) then
	FF_SetControl "ImageBackup","strPathOriginal",strPathOriginal
else
	FF_SetControl "ImageBackup","strPathOriginal",FF_GetImagePath
end if

FF_SetControl "ImageBackup","flag_Auto",flag_Auto
FF_SetControl "ImageBackup","flag_Struktur",flag_Struktur

if len(strDrive) > 0 and instr(Laufwerke,strDrive) > 0 then
	FF_SetControl "ImageBackup","strDrive",strDrive
end if

FF_SetControl "ImageBackup","strPathKopie",strPathKopie

'### Alarmmeldungen

if not objFS.FolderExists(strPathOriginal) then
	msgbox "Der Quellordner " & vbNewLine & "  " & strPathOriginal & vbNewLine &_
		"existiert nicht!" & vbNewLine & "Ersatzweise wird das aktuelle Verzeichnis eingetragen.",vbExclamation,"Hinweis"
end if

if instr(Laufwerke,strDrive) = 0 then
	msgbox "Das Laufwerk " & strDrive & vbNewLine &_
		"wurde nicht gefunden!",vbExclamation,"Hinweis"
end if

End Sub
'------------------------------------------------------------------------
Sub RegistrySchreiben
'Schlsselname = HKEY_CURRENT_USER\Software\Joachim Koopmann Software\FixFoto\Script\ & RegSchluessel

call WerteAuslesen

'Werte schreiben
FF_WriteProfile RegSchluessel,"strPathOriginal",strPathOriginal
FF_WriteProfile RegSchluessel,"flag_Auto",flag_Auto
FF_WriteProfile RegSchluessel,"flag_Struktur",flag_Struktur
FF_WriteProfile RegSchluessel,"strDrive",strDrive
FF_WriteProfile RegSchluessel,"strPathKopie",strPathKopie

End Sub
'------------------------------------------------------------------------
'************************************************************************
'Delete-Funktion
'************************************************************************
'------------------------------------------------------------------------
Sub KillSkript
'Skript mit allen Komponenten entfernen

Dim SPfad

'### FixFotoScript.ini bereinigen
TempFile = FF_GetFFPathEx(1) & "\FixFotoScript.ini"
if objFS.FileExists(TempFile) then
	call IniBereinigung
	if Abbruch then exit sub
else
	msgbox 	"Die Datei FixFotoScript.ini konnte nicht gefunden werden." & vbNewLine & _
			"Der Lschvorgang wird abgebrochen!",vbExclamation,"Hinweis"
	exit sub
end if

'### zip-Archiv auf Wunsch sichern, dann lschen
TempFile = FF_GetScriptPath & "\" & objFS.GetBaseName(Skriptname) & ".zip"
if objFS.FileExists(TempFile) then
	Antwort = msgbox("Soll das Archiv " & objFS.GetBaseName(Skriptname) & ".zip" & _
			 " gesichert werden?",vbYesNo + vbQuestion,"Sicherheitskopie")
	if Antwort = vbYes then
		SPfad = FF_EnterPath(true,"Wo soll die Sicherheitskopie angelegt werden?")
		if SPfad = "" then
			msgbox "Das Lschen des Skripts wird abgebrochen!",vbExclamation,"Hinweis"
			exit sub
		end if
		SPfad = SPfad & "\" & objFS.GetBaseName(Skriptname) & ".zip"
		objFS.CopyFile Tempfile, SPfad, true
	end if
	objFS.DeleteFile TempFile,true
end if

'### Skriptdatei lschen
TempFile = FF_GetScriptPath & "\" & Skriptname
objFS.DeleteFile TempFile,true

'### evtl. beigepackten Ordner lschen
TempFile = FF_GetScriptPath & "\MeckiScript\ImageBackup"
if objFS.FolderExists(TempFile) then objFS.DeleteFolder TempFile,true

'### ggf. Registry-Daten lschen
'TempFile = "HKEY_CURRENT_USER\Software\Joachim Koopmann Software\FixFoto\Script\" & RegSchluessel & "\"
'WshShell.RegDelete(TempFile)

'### Fertigmeldung
msgbox "Das Skript" & Skriptname & vbNewLine & "wurde mit allen Komponenten gelscht!" & vbNewLine & _
	   "Es wird empfohlen die 'Automatische Skriptkonfiguration' aufzurufen.",vbExclamation,"Hinweis"

End Sub
'------------------------------------------------------------------------
Sub IniBereinigung
'FixFotoScript.ini - Bereinigung

Dim Zeile, zNr,Anfang, Ende, DateiTmp, ZielZeile, tmpString
DateiTmp =  FF_GetFFPathEx(1) & "\FixFotoScript.tmp"

'### Wo befindet sich der Eintrag zum Skript?
Set Zeile = objFS.OpenTextFile(TempFile, ForReading)

zNr = 0
do Until Zeile.AtEndOfStream
	tmpString = Zeile.ReadLine
	zNr = zNr + 1
	if tmpString = "[Script]" then Anfang = zNr
	if tmpString = "File=.\" & Skriptname then
		Ende = zNr
		exit do	
	end if
loop 
Zeile.Close

if Ende <= Anfang then
	msgbox 	"Die Datei FixFotoScript.ini konnte nicht bereinigt werden." & vbNewLine & _
		"Der Lschvorgang wird abgebrochen!",vbExclamation,"Hinweis"
	Abbruch = true
	exit sub
end if

'### Zeilen ohne skriptbezogene Zeilen in DateiTmp bertragen
Set Zeile = objFS.OpenTextFile(TempFile, ForReading)
Set ZielZeile  = objFS.CreateTextFile(DateiTmp, True)

zNr = 0
do Until Zeile.AtEndOfStream
	tmpString = Zeile.ReadLine
	zNr = zNr + 1
	if zNr < Anfang or zNr > Ende then
		ZielZeile.WriteLine tmpString
	end if
loop 

Zeile.Close
ZielZeile.Close

'### Umkopieren von DateiTmp nach FixFotoScript.ini (TempFile)
objFS.CopyFile DateiTmp, TempFile, True
objFS.DeleteFile (DateiTmp)

End Sub
'------------------------------------------------------------------------
'************************************************************************
'Kurzbeschreibung
'************************************************************************
'------------------------------------------------------------------------
Sub KurzbeschreibungAnzeigen

Dim Taste
Dim TBreite	: TBreite = LaengeMax * 2.6 + 10
const THoehe = 180

const ES_READONLY = 2048
const WS_VSCROLL = 2097152

FF_AddDialog "Kurzbeschreibung",TBreite,THoehe
FF_AddControl "Kurzbeschreibung","Beschreibung","EDIT",0,0,TBreite,THoehe-20
FF_SetControlStyle "Kurzbeschreibung","Beschreibung", ES_READONLY + WS_VSCROLL
FF_AddControl "Kurzbeschreibung","OK","BUTTON",TBreite/2-15,THoehe-15,30,10

FF_SetControl "Kurzbeschreibung","Beschreibung",Kurzbeschreibung

do
	Taste = FF_ShowDialog("Kurzbeschreibung")
	if Taste = "CANCEL" or Taste = "OK" then exit do
loop
call FF_CloseDialog("Kurzbeschreibung")

End Sub
'------------------------------------------------------------------------
Sub KurzbeschreibungLesen
on error resume next
Const ForReading = 1
Dim Skript, Anfang, Ende, Zeile

Set Skript = objFS.OpenTextFile(FF_GetScriptPath & "\" & SkriptName, ForReading, False)

if err.number = 53 then
	msgbox "Wahrscheinlich wurde der Originalskriptname " & """" & SkriptName & """" & " abgendert." & vbNewLine & _
		   "Die Kurzbeschreibung kann daher nicht angezeigt werden.",vbExclamation,"Meldung"
	Abbruch = True
	exit sub
end if

Do While Skript.AtEndOfStream <> True
	Zeile = Skript.ReadLine
	
	if instr(Zeile,"** Ende Kurzbeschreibung") > 0 then Ende = true
	if Ende then exit do
	
	if Anfang then Kurzbeschreibung = Kurzbeschreibung & mid(Zeile,2) & vbNewLine
	if len(Zeile)-1 > LaengeMax then LaengeMax = len(Zeile)-1
	
	if instr(Zeile,"** Start Kurzbeschreibung") > 0 then Anfang = true
	
Loop
	
if Anfang and Ende then KBvorhanden = true
Skript.Close

End Sub
'------------------------------------------------------------------------
'************************************************************************
'Update-Funktion
'************************************************************************
Sub Update
Dim Antwort

call Versionsermittlung_url

if not Abbruch then
	SkriptV_Zahl = Replace(SkriptVersion,".","") *1
	
	if UrlV_Zahl = SkriptV_Zahl then
		msgbox "Die Version " & SkriptVersion & " ist aktuell!",vbInformation,"Versionsmeldung"
	elseif SkriptV_Zahl < UrlV_Zahl then
		Antwort = msgbox("Die Skriptversion " & SkriptVersion & " ist nicht mehr aktuell!" & vbNewLine &_
				"Es steht die Version " & UrlVersion & " zum Download bereit." & vbNewLine & vbNewLine & _
				"Soll die Downloadseite aufgerufen werden?",vbQuestion + vbYesNo,"Versionsmeldung")
		if Antwort = vbYes then
			on error resume next
			WshShell.Run SkriptUrl,1,false
			if err <> 0 then
				msgbox "Die Downloadseite wurde nicht gefunden!",vbExclamation,"Hinweis"
			else
				SkriptEnde = true
			end if
			on error goto 0
		end if
	else
		msgbox 	"Es liegt ein Fehler in den Versionsangaben vor." & vbNewLine &_
				"Die Skriptversion " & SkriptVersion & " ist grer als die auf" & vbNewLine &_
				"der Downloadseite angegebene Version " & UrlVersion & ".",vbInformation,"Fehlermeldung"
	end if
end if

End Sub
'------------------------------------------------------------------------
Sub Versionsermittlung_url

Dim Inhalt,Start,Zeit,pos1,pos2,n
Dim xml, Fehler
Fehler = false

on error resume next
Set xml = CreateObject("Microsoft.XMLHTTP")
xml.Open "Get",SkriptUrl,false
xml.Send

Inhalt = xml.responseText

if err > 0 then
	Fehler = true
else
	'Version suchen und anzeigen
	pos1 = instr(Inhalt,"Version:")
	if pos1 > 0 then
	
		pos2 = instr(pos1 ,Inhalt,"<br")
		if pos2 > 0 then
			UrlVersion = trim(mid(Inhalt,pos1,pos2-pos1))
			'Msgbox "Aktuell ist die " & UrlVersion,vbInformation,"Information"
		else
			Fehler = true
		end if
	else
		Fehler = true
	end if
end if

Set xml = nothing

if Fehler then
	msgbox  "Die Versionsangabe wurde nicht gefunden!" & vbNewLine &_
			"Mglicherweise funktioniert der Internetaufruf nicht." & vbNewLine & vbNewLine & _
			"Am besten nochmal probieren!",vbExclamation,"Fehlermeldung"
	Abbruch = true
	exit sub
end if

'Version als Zahl
pos1 = instr(UrlVersion," ")
if pos1 = 0 then pos1 = 7
UrlVersion = trim(mid(UrlVersion,pos1+1))
UrlV_Zahl = Replace(UrlVersion,".","") *1

End Sub
'------------------------------------------------------------------------
'************************************************************************
'Allgemein
'************************************************************************
'------------------------------------------------------------------------
Function Version_OK(MinVersion)
dim VersNr

 VersNr=split(MinVersion,".")
 Version_OK=FF_GetMajor * 1000 + FF_GetMinor + FF_GetBuild / 1000 >=_
     		VersNr(0) * 1000 + VersNr(1) + VersNr(2) / 1000

End Function
'------------------------------------------------------------------------
Function Laufwerke
'gltige Laufwerke ermitteln
Dim LW,d,vhdn

For Each LW In objfs.Drives
	Set d = objfs.GetDrive(LW.DriveLetter)
	If d.IsReady And d.DriveType > 0 And d.DriveType < 4 Then
		If Len(vhdN) = 0 Then
			vhdN = LW.DriveLetter & ":  [" & LW.VolumeName & "]"
		Else
			vhdN = vhdN & "," & LW.DriveLetter & ":  [" & LW.VolumeName & "]"
		End If
	End If
Next
Laufwerke = vhdn

End Function
'------------------------------------------------------------------------
Function Zeichentest_Ok (ByVal DName)
'Test auf unzulssige Zeichen bei Datei- und Ordnernamen
Dim zArray,z

'zArray = Split("\,/,:,*,?,<,>,|," & Chr(34),",")	'mit Slash
zArray = Split("/,:,*,?,<,>,|," & Chr(34),",")		'ohne Slash
For Each z In zArray
	If InStr(DName,z) > 0 Then
		Zeichentest_Ok = False
		Exit Function
	End If
Next
Zeichentest_Ok = True

End Function
'------------------------------------------------------------------------
Function SpeicherGroesse(ByVal Bytes)
'Dimensionieren von Speichergren
Dim Hochzahl, Dimension
Bytes = Bytes *1
	If Bytes > 1024 ^ 3 Then
		Hochzahl = 3 : Dimension = " GB"
	ElseIf Bytes > 1024 ^ 2 Then
		Hochzahl = 2 : Dimension = " MB"
	ElseIf Bytes > 1024 ^ 1 Then
			Hochzahl = 1 : Dimension = " KB"		
	ElseIf Bytes > 1024 ^ 0 Then
			Hochzahl = 0 : Dimension = " Byte"
	End If
SpeicherGroesse =  FormatNumber(Bytes / 1024 ^ Hochzahl, 2, - 2, - 2, - 2) & Dimension
End Function
'------------------------------------------------------------------------
Sub ShowSubfolderCount(ByVal myfolder)
'AnzGes = Anzahl der Dateien in myfolder incl. Unterordner
Dim subfolder
	AnzGes = AnzGes + myfolder.files.count
	For Each subfolder In myfolder.subfolders
		ShowSubfolderCount subfolder
	Next
End Sub
'------------------------------------------------------------------------
Sub CreateFolder(ByVal strPath)
'Erzeugen des Ordnerpfads strPath incl. aller Unterordner
Dim arrfeld, strOrdner,x
	arrfeld = Split(strPath, "\")
	strOrdner = arrfeld(0) & "\"
	For x = 1 To UBound(arrfeld)
		strOrdner = objfs.BuildPath(strOrdner, arrfeld(x))
		If Not objfs.FolderExists(strOrdner) Then
			objfs.CreateFolder strOrdner
		End If
	Next
End Sub
'------------------------------------------------------------------------
